@@ -29,7 +29,7 @@ from socket import AF_INET, AF_INET6
 from threading import Thread
 from tkinter import E, Frame, N, PhotoImage, S, TclError, Tk, Toplevel, W, font
 
-from pygnssutils import GNSSMQTTClient, GNSSNTRIPClient
+from pygnssutils import GNSSMQTTClient, GNSSNTRIPClient, MQTTMessage
 from pygnssutils.socket_server import ClientHandler, SocketServer
 from pyubx2 import NMEA_PROTOCOL, RTCM3_PROTOCOL, UBX_PROTOCOL, protocol
 from serial import SerialException, SerialTimeoutException
@@ -51,8 +51,10 @@ from pygpsclient.globals import (
     GNSS_EOF_EVENT,
     GNSS_ERR_EVENT,
     GNSS_EVENT,
+    GNSS_TIMEOUT_EVENT,
     GUI_UPDATE_INTERVAL,
     ICON_APP,
+    MQTT_PROTOCOL,
     NOPORTS,
     NTRIP_EVENT,
     OKCOL,
@@ -72,6 +74,7 @@ from pygpsclient.strings import (
     CONFIGERR,
     DLGSTOPRTK,
     ENDOFFILE,
+    INACTIVE_TIMEOUT,
     INTROTXTNOPORTS,
     LOADCONFIGBAD,
     LOADCONFIGNONE,
@@ -130,6 +133,9 @@ class App(Frame):
         self.mqttclientregion = kwargs.pop(
             "mqttclientregion", getenv("MQTTCLIENTREGION", "eu")
         )
+        self.mqttclientmode = int(
+            kwargs.pop("mqttclientmode", getenv("MQTTCLIENTMODE", "0"))
+        )
         self.ntripcaster_user = kwargs.pop(
             "ntripuser", getenv("PYGPSCLIENT_USER", "anon")
         )
@@ -342,6 +348,7 @@ class App(Frame):
 
         self.__master.bind(GNSS_EVENT, self.on_gnss_read)
         self.__master.bind(GNSS_EOF_EVENT, self.on_gnss_eof)
+        self.__master.bind(GNSS_TIMEOUT_EVENT, self.on_gnss_timeout)
         self.__master.bind(GNSS_ERR_EVENT, self.on_stream_error)
         self.__master.bind(NTRIP_EVENT, self.on_ntrip_read)
         self.__master.bind(SPARTN_EVENT, self.on_spartn_read)
@@ -529,6 +536,9 @@ class App(Frame):
             spartnsettings["region"] = self.saved_config.get(
                 "mgttclientregion_s", self.mqttclientregion
             )
+            spartnsettings["mode"] = self.saved_config.get(
+                "mgttclientmode_n", self.mqttclientmode
+            )
             spartnsettings["topic_ip"] = self.saved_config.get("mgttclienttopicip_b", 1)
             spartnsettings["topic_mga"] = self.saved_config.get(
                 "mgttclienttopicmga_b", 1
@@ -720,7 +730,21 @@ class App(Frame):
             False  # turn off socket server
         )
         self.conn_status = DISCONNECTED
-        self.set_status(ENDOFFILE)
+        self.set_status(ENDOFFILE, "red")
+
+    def on_gnss_timeout(self, event):  # pylint: disable=unused-argument
+        """
+        EVENT TRIGGERED
+        Action on <<sgnss_timeout>> event - stream inactivity timeout.
+
+        :param event event: <<gnss_timeout>> event
+        """
+
+        self.frm_settings.frm_socketserver.socketserving = (
+            False  # turn off socket server
+        )
+        self.conn_status = DISCONNECTED
+        self.set_status(INACTIVE_TIMEOUT, "red")
 
     def on_ntrip_read(self, event):  # pylint: disable=unused-argument
         """
@@ -737,7 +761,9 @@ class App(Frame):
                     if self.conn_status == CONNECTED:
                         self.gnss_outqueue.put(raw_data)
                     self.process_data(raw_data, parsed_data, "NTRIP>>")
-                else:  # e.g. NMEA GGA sentence sent to NTRIP server
+                elif (
+                    protocol(raw_data) == NMEA_PROTOCOL
+                ):  # e.g. NMEA GGA sentence sent to NTRIP server
                     self.process_data(raw_data, parsed_data, "NTRIP<<")
             self.ntrip_inqueue.task_done()
         except Empty:
@@ -816,7 +842,9 @@ class App(Frame):
 
         protfilter = settings["protocol_n"]
         msgprot = protocol(raw_data)
-        if isinstance(parsed_data, str):  # error message rather than parsed data
+        if isinstance(parsed_data, MQTTMessage):
+            msgprot = MQTT_PROTOCOL
+        elif isinstance(parsed_data, str):
             marker = "WARNING  "
             msgprot = 0
 
@@ -826,6 +854,8 @@ class App(Frame):
             self.nmea_handler.process_data(raw_data, parsed_data)
         elif msgprot == RTCM3_PROTOCOL and msgprot & protfilter:
             self.rtcm_handler.process_data(raw_data, parsed_data)
+        elif msgprot == MQTT_PROTOCOL:
+            pass
 
         # update console if visible
         if widget_state["Console"][VISIBLE]:
