@@ -42,8 +42,6 @@ from requests import ConnectTimeout, HTTPError, RequestException, get
 from pygpsclient.globals import (
     BGCOL,
     DLGTGPX,
-    GPXLIMIT,
-    GPXMAPURL,
     HOME,
     ICON_EXIT,
     ICON_LOAD,
@@ -55,14 +53,14 @@ from pygpsclient.globals import (
     KPH2MPH,
     KPH2MPS,
     M2FT,
-    MAPQTIMEOUT,
     POPUP_TRANSIENT,
     READONLY,
     UI,
     UIK,
     UMK,
 )
-from pygpsclient.helpers import haversine, mapq_compress
+from pygpsclient.helpers import haversine
+from pygpsclient.mapquest import GPXLIMIT, GPXMAPURL, MAPQTIMEOUT, mapq_compress
 from pygpsclient.strings import (
     DLGGPXERROR,
     DLGGPXLOAD,
@@ -104,6 +102,7 @@ class GPXViewerDialog(Toplevel):
         self.mheight = int(self.height * 0.75)
         self.pheight = int(self.height * 0.25)
         self._zoom = IntVar()
+        self._maptype = StringVar()
         zoom = int(kwargs.get("zoom", 12))
         self._zoom.set(zoom)
         self._info = []
@@ -157,6 +156,15 @@ class GPXViewerDialog(Toplevel):
             textvariable=self._zoom,
             state=READONLY,
         )
+        self._lbl_maptype = Label(self._frm_controls, text="Map Type")
+        self._spn_maptype = Spinbox(
+            self._frm_controls,
+            values=("map", "sat"),
+            width=5,
+            wrap=True,
+            textvariable=self._maptype,
+            state=READONLY,
+        )
         self._btn_redraw = Button(
             self._frm_controls,
             image=self._img_redraw,
@@ -196,13 +204,25 @@ class GPXViewerDialog(Toplevel):
             padx=3,
             pady=3,
         )
-        self._btn_redraw.grid(
+        self._lbl_maptype.grid(
             column=3,
             row=1,
             padx=3,
             pady=3,
         )
-        self._btn_exit.grid(column=4, row=1, padx=3, pady=3, sticky=E)
+        self._spn_maptype.grid(
+            column=4,
+            row=1,
+            padx=3,
+            pady=3,
+        )
+        self._btn_redraw.grid(
+            column=5,
+            row=1,
+            padx=3,
+            pady=3,
+        )
+        self._btn_exit.grid(column=6, row=1, padx=3, pady=3, sticky=E)
 
         self.grid_columnconfigure(0, weight=1)
         self.grid_rowconfigure(0, weight=3)
@@ -385,7 +405,9 @@ class GPXViewerDialog(Toplevel):
         """
         # pylint: disable=unused-variable
 
-        mqapikey = self.__app.config.get("mqapikey", "")
+        mqapikey = self.__app.frm_settings.config.get(
+            "mqapikey_s", self.__app.frm_settings.config.get("mqapikey", "")
+        )
         lat1, lon1, _, _, _ = self._track[0]  # start point, labelled 1
         lat2, lon2, _, _, _ = self._track[-1]  # end point, labelled 2
         points = []
@@ -407,6 +429,7 @@ class GPXViewerDialog(Toplevel):
         # seems to be bug in MapQuest API which causes error
         # if scalebar displayed at maximum zoom
         scalebar = "true" if self._zoom.get() < 20 else "false"
+        maptype = self._maptype.get()  # "map" or "sat"
 
         try:
             url = GPXMAPURL.format(
@@ -421,6 +444,7 @@ class GPXViewerDialog(Toplevel):
                 "ff00ff",
                 tstr,
                 scalebar,
+                maptype,
             )
             response = get(url, timeout=MAPQTIMEOUT)
             response.raise_for_status()  # raise Exception on HTTP error
@@ -518,7 +542,7 @@ class GPXViewerDialog(Toplevel):
         )
 
         # plot speed (yR) axis grid
-        for spd in range(0, maxs, int(maxs / 5)):
+        for spd in range(0, int(maxs), int(maxs / 5)):
             if spd is None:
                 continue
             x1, y1 = self._get_point(maxs, maxx, spd, maxx)
@@ -601,7 +625,7 @@ class GPXViewerDialog(Toplevel):
         """
 
         settings = self.__app.frm_settings.config
-        units = settings["units"]
+        units = settings["units_s"]
 
         if units == UI:
             dst_u = "miles"
