@@ -1,4 +1,6 @@
 """
+map_frame.py
+
 Mapview frame class for PyGPSClient application.
 
 This handles a frame containing a location map downloaded via a MapQuest API.
@@ -25,18 +27,20 @@ from PIL import Image, ImageTk
 from requests import ConnectionError as ConnError
 from requests import ConnectTimeout, RequestException, get
 
-from pygpsclient.globals import (
-    BGCOL,
-    ICON_POS,
-    IMG_WORLD,
+from pygpsclient.globals import BGCOL, ICON_POS, IMG_WORLD, WIDGETU2
+from pygpsclient.mapquest import (
     MAP_UPDATE_INTERVAL,
     MAPQTIMEOUT,
     MAPURL,
+    MAX_ZOOM,
     MIN_UPDATE_INTERVAL,
-    WIDGETU2,
+    MIN_ZOOM,
 )
 from pygpsclient.strings import NOWEBMAPCONN, NOWEBMAPFIX, NOWEBMAPHTTP, NOWEBMAPKEY
 
+ZOOMCOL = "red"
+ZOOMEND = "lightgray"
+
 
 class MapviewFrame(Frame):
     """
@@ -63,10 +67,13 @@ class MapviewFrame(Frame):
         self._img = None
         self._marker = None
         self._last_map_update = 0
-        # self._map_update_interval = MAP_UPDATE_INTERVAL
+        self._resize_font = font.Font(size=min(int(self.height / 5), 30))
+        self._resize_font_height = self._resize_font.metrics("linespace")
+        self._resize_font_width = self._resize_font.measure("+")
+        self._zoom = int((MAX_ZOOM - MIN_ZOOM) / 2)
+        self._lastmaptype = ""
         self._body()
-
-        self.bind("<Configure>", self._on_resize)
+        self._attach_events()
 
     def _body(self):
         """
@@ -75,8 +82,64 @@ class MapviewFrame(Frame):
 
         self.grid_columnconfigure(0, weight=1)
         self.grid_rowconfigure(0, weight=1)
-        self.can_mapview = Canvas(self, width=self.width, height=self.height, bg=BGCOL)
-        self.can_mapview.grid(column=0, row=0, sticky=(N, S, E, W))
+        self._can_mapview = Canvas(self, width=self.width, height=self.height, bg=BGCOL)
+        self._can_mapview.grid(column=0, row=0, sticky=(N, S, E, W))
+
+    def _attach_events(self):
+        """
+        Bind events to frame.
+        """
+
+        self.bind("<Configure>", self._on_resize)
+        self._can_mapview.bind("<Double-Button-1>", self.on_refresh)  # double-click
+        self._can_mapview.bind("<Button-1>", self.on_zoom)  # left-click
+        self._can_mapview.bind("<Button-2>", self.on_zoom)  # right click Posix
+        self._can_mapview.bind("<Button-3>", self.on_zoom)  # right-click Windows
+
+    def init_frame(self):
+        """
+        Initialise map.
+        """
+
+        self._zoom = self.__app.frm_settings.config.get("mapzoom", 10)
+
+    def on_refresh(self, event):  # pylint: disable=unused-argument
+        """
+        Trigger refresh of web map.
+
+        :param event: event
+        """
+
+        self._last_map_update = 0
+
+    def on_zoom(self, event):  # pylint: disable=unused-argument
+        """
+        Trigger zoom in or out.
+
+        Left click (event.num = 1) increments zoom by 1.
+        Right click (event.num = 2/3) increments zoom to maximum extent.
+
+        :param event: event
+        """
+
+        refresh = False
+        w, h = self.width, self.height
+        fw, fh = self._resize_font_width, self._resize_font_height
+        # zoom out (-) if not already at min
+        if w > event.x > w - 2 - fw and h > event.y > h - fh:
+            if self._zoom > MIN_ZOOM:
+                zinc = -1 if event.num == 1 else MIN_ZOOM - self._zoom
+                refresh = True
+        # zoom in (+) if not already at max
+        elif w > event.x > w - 2 - fw and h - fh > event.y > h - fh * 2:
+            if self._zoom < MAX_ZOOM:
+                zinc = 1 if event.num == 1 else MAX_ZOOM - self._zoom
+                refresh = True
+
+        if refresh:
+            self._zoom += zinc
+            self.__app.frm_settings.mapzoom.set(self._zoom)
+            self.on_refresh(event)
 
     def update_frame(self):
         """
@@ -86,11 +149,6 @@ class MapviewFrame(Frame):
         lat = self.__app.gnss_status.lat
         lon = self.__app.gnss_status.lon
         hacc = self.__app.gnss_status.hacc
-        settings = self.__app.frm_settings.config
-        if settings["webmap"]:
-            static = False
-        else:
-            static = True
 
         # if no valid position, display warning message
         # fix = kwargs.get("fix", 0)
@@ -104,12 +162,13 @@ class MapviewFrame(Frame):
             self._disp_error(NOWEBMAPFIX)
             return
 
-        if static:
+        maptype = self.__app.frm_settings.config.get("maptype_s", "world")
+        if maptype == "world":
             self._draw_static_map(lat, lon)
         else:
             if hacc is None or hacc == "":
                 hacc = 0
-            self._draw_web_map(lat, lon, hacc)
+            self._draw_web_map(lat, lon, hacc, maptype)
 
     def _draw_static_map(self, lat: float, lon: float):
         """
@@ -118,41 +177,56 @@ class MapviewFrame(Frame):
         :param float lat: latitude
         :param float lon: longitude
         """
+        # pylint: disable=no-member
 
         OFFSET_X = 0
         OFFSET_Y = 0
 
+        self._lastmaptype = "world"
         w, h = self.width, self.height
-        self.can_mapview.delete("all")
+        self._can_mapview.delete("all")
         self._img = ImageTk.PhotoImage(
-            Image.open(IMG_WORLD).resize((w, h), Image.ANTIALIAS)
+            Image.open(IMG_WORLD).resize((w, h))  # , Image.ANTIALIAS)
         )
         self._marker = ImageTk.PhotoImage(Image.open(ICON_POS))
-        self.can_mapview.create_image(0, 0, image=self._img, anchor=NW)
+        self._can_mapview.create_image(0, 0, image=self._img, anchor=NW)
         x = (w / 2) + int((lon * (w / 360))) + OFFSET_X
         y = (h / 2) - int((lat * (h / 180))) + OFFSET_Y
-        self.can_mapview.create_image(x, y, image=self._marker, anchor=S)
+        self._can_mapview.create_image(x, y, image=self._marker, anchor=S)
 
-    def _draw_web_map(self, lat: float, lon: float, hacc: float):
+    def _draw_web_map(self, lat: float, lon: float, hacc: float, maptype: str = "map"):
         """
         Draw scalable web map via MapQuest API
 
         :param float lat: latitude
         :param float lon: longitude
         :param float hacc: horizontal accuracy
+        :param str maptype: "map" or "sat"
         """
 
         sc = "NO CONNECTION"
         msg = ""
 
-        mqapikey = self.__app.app_config.get("mqapikey", getenv("mqapikey", ""))
+        if maptype != self._lastmaptype:
+            self._lastmaptype = maptype
+            self.reset_map_refresh()
+
+        mqapikey = self.__app.frm_settings.config.get(
+            "mqapikey_s",
+            self.__app.frm_settings.config.get("mqapikey", getenv("mqapikey", "")),
+        )
         if mqapikey == "":
             self._disp_error(NOWEBMAPKEY)
             return
-        map_update_interval = self.__app.app_config.get(
-            "mapupdateinterval", MAP_UPDATE_INTERVAL
+        map_update_interval = max(
+            self.__app.frm_settings.config.get(
+                "mapupdateinterval_n",
+                self.__app.frm_settings.config.get(
+                    "mapupdateinterval", MAP_UPDATE_INTERVAL
+                ),
+            ),
+            MIN_UPDATE_INTERVAL,
         )
-        map_update_interval = max(map_update_interval, MIN_UPDATE_INTERVAL)
 
         now = time()
         if now - self._last_map_update < map_update_interval:
@@ -162,7 +236,7 @@ class MapviewFrame(Frame):
             return
         self._last_map_update = now
 
-        url = self._format_url(mqapikey, lat, lon, hacc)
+        url = self._format_url(mqapikey, maptype, lat, lon, hacc)
 
         try:
             response = get(url, timeout=MAPQTIMEOUT)
@@ -171,9 +245,10 @@ class MapviewFrame(Frame):
             if sc == "OK":
                 img_data = response.content
                 self._img = ImageTk.PhotoImage(Image.open(BytesIO(img_data)))
-                self.can_mapview.delete("all")
-                self.can_mapview.create_image(0, 0, image=self._img, anchor=NW)
-                self.can_mapview.update_idletasks()
+                self._can_mapview.delete("all")
+                self._can_mapview.create_image(0, 0, image=self._img, anchor=NW)
+                self._draw_zoom()
+                self._can_mapview.update_idletasks()
                 return
         except (ConnError, ConnectTimeout):
             msg = NOWEBMAPCONN
@@ -189,16 +264,51 @@ class MapviewFrame(Frame):
         :param int wait: wait time in seconds
         """
 
-        self.can_mapview.create_oval((5, 5, 20, 20), fill="#616161", outline="")
-        self.can_mapview.create_arc(
+        self._can_mapview.create_oval((5, 5, 20, 20), fill="#616161", outline="")
+        self._can_mapview.create_arc(
             (5, 5, 20, 20), start=90, extent=wait, fill="#ffffff", outline=""
         )
 
-    def _format_url(self, mqapikey: str, lat: float, lon: float, hacc: float):
+    def _draw_zoom(self):
+        """
+        Draw +/- zoom icons.
+        """
+
+        w, h = self.width, self.height
+        fw, fh = self._resize_font_width, self._resize_font_height
+        self._can_mapview.create_text(
+            w - 2 - fw / 2,
+            h - 2 - fh,
+            text="+",
+            font=self._resize_font,
+            fill=ZOOMCOL if self._zoom < MAX_ZOOM else ZOOMEND,
+            anchor="s",
+        )
+        self._can_mapview.create_text(
+            w - 2 - fw / 2,
+            h - 2 - fh / 1.2,
+            text=self._zoom,
+            fill=ZOOMCOL,
+            font=font.Font(size=8),
+            # anchor="e",
+        )
+        self._can_mapview.create_text(
+            w - 2 - fw / 2,
+            h - 2,
+            text="−",
+            font=self._resize_font,
+            fill=ZOOMCOL if self._zoom > MIN_ZOOM else ZOOMEND,
+            anchor="s",
+        )
+
+    def _format_url(
+        self, mqapikey: str, maptype: str, lat: float, lon: float, hacc: float
+    ):
         """
         Formats URL for web map download.
 
         :param str mqapikey: MapQuest API key
+        :param str maptype: "map" or "sat"
         :param float lat: latitude
         :param float lon: longitude
         :param float hacc: horizontal accuracy
@@ -208,8 +318,7 @@ class MapviewFrame(Frame):
 
         w, h = self.width, self.height
         radius = str(hacc / 1000)  # km
-        settings = self.__app.frm_settings.config
-        zoom = settings["mapzoom"]
+        zoom = self._zoom
         # seems to be bug in MapQuest API which causes error
         # if scalebar displayed at maximum zoom
         scalebar = "true" if zoom < 20 else "false"
@@ -225,6 +334,7 @@ class MapviewFrame(Frame):
             lat,
             lon,
             scalebar,
+            maptype,
         )
 
     def _disp_error(self, msg):
@@ -237,8 +347,8 @@ class MapviewFrame(Frame):
         w, h = self.width, self.height
         resize_font = font.Font(size=min(int(w / 20), 14))
 
-        self.can_mapview.delete("all")
-        self.can_mapview.create_text(
+        self._can_mapview.delete("all")
+        self._can_mapview.create_text(
             w / 2,
             h / 2,
             text=msg,
@@ -262,6 +372,9 @@ class MapviewFrame(Frame):
         """
 
         self.width, self.height = self.get_size()
+        self._resize_font = font.Font(size=min(int(self.height / 5), 30))
+        self._resize_font_height = self._resize_font.metrics("linespace")
+        self._resize_font_width = self._resize_font.measure("+")
 
     def get_size(self):
         """
@@ -272,6 +385,6 @@ class MapviewFrame(Frame):
         """
 
         self.update_idletasks()  # Make sure we know about any resizing
-        width = self.can_mapview.winfo_width()
-        height = self.can_mapview.winfo_height()
+        width = self._can_mapview.winfo_width()
+        height = self._can_mapview.winfo_height()
         return (width, height)
