@@ -1,7 +1,12 @@
 """
-UBX Protocol handler
+ubx_handler.py
 
-Uses pyubx2 library for parsing
+UBX Protocol handler - handles all incoming UBX messages
+
+Parses individual UBX messages (using pyubx2 library)
+and adds selected attribute values to the app.gnss_status
+data dictionary. This dictionary is then used to periodically
+update the various user-selectable widget panels.
 
 Created on 30 Sep 2020
 
@@ -13,7 +18,8 @@ Created on 30 Sep 2020
 from pyubx2 import UBXMessage, itow2utc
 
 from pygpsclient.globals import DLGTSPARTN, DLGTUBX, GLONASS_NMEA
-from pygpsclient.helpers import corrage2int, fix2desc, svid2gnssid
+from pygpsclient.helpers import corrage2int, fix2desc, ned2vector, svid2gnssid
+from pygpsclient.widget_state import VISIBLE, WDGSPECTRUM, WDGSYSMON
 
 
 class UBXHandler:
@@ -50,19 +56,25 @@ class UBXHandler:
             return
 
         if parsed_data.identity[0:3] in ("ACK", "CFG"):
-            self._update_ubxconfig(parsed_data)
+            self._process_ACK(parsed_data)
         elif parsed_data.identity in ("MON-VER", "MON-HW"):
-            self._update_ubxconfig(parsed_data)
+            self._process_MONVER(parsed_data)
         elif parsed_data.identity in ("NAV-POSLLH", "NAV-HPPOSLLH"):
             self._process_NAV_POSLLH(parsed_data)
         elif parsed_data.identity in ("NAV-PVT", "NAV2-PVT"):
             self._process_NAV_PVT(parsed_data)
+        elif parsed_data.identity == "NAV-PVAT":
+            self._process_NAV_PVAT(parsed_data)
+        elif parsed_data.identity == "NAV-RELPOSNED":
+            self._process_NAV_RELPOSNED(parsed_data)
         elif parsed_data.identity == "NAV-VELNED":
             self._process_NAV_VELNED(parsed_data)
         elif parsed_data.identity in ("NAV-SAT", "NAV2-SAT"):
             self._process_NAV_SAT(parsed_data)
         elif parsed_data.identity in ("NAV-STATUS", "NAV2-STATUS)"):
             self._process_NAV_STATUS(parsed_data)
+        elif parsed_data.identity == "NAV-SVIN":
+            self._process_NAV_SVIN(parsed_data)
         elif parsed_data.identity == "NAV-SVINFO":
             self._process_NAV_SVINFO(parsed_data)
         elif parsed_data.identity == "NAV-SOL":
@@ -75,12 +87,18 @@ class UBXHandler:
             self._process_RXM_RTCM(parsed_data)
         elif parsed_data.identity == "MON-SPAN":
             self._process_MON_SPAN(parsed_data)
+        elif parsed_data.identity == "MON-SYS":
+            self._process_MON_SYS(parsed_data)
+        elif parsed_data.identity == "MON-COMMS":
+            self._process_MON_COMMS(parsed_data)
+        elif parsed_data.identity == "RXM-PMP":
+            self._process_RXM_PMP(parsed_data)
         elif parsed_data.identity == "RXM-SPARTN-KEY":
             self._process_RXM_SPARTN_KEY(parsed_data)
 
-    def _update_ubxconfig(self, msg: UBXMessage):
+    def _process_ACK(self, msg: UBXMessage):
         """
-        Update UBX Config dialog status.
+        Process ACK-ACK & ACK-NAK sentences and CFG poll responses.
 
         :param UBXMessage msg: UBX config message
         """
@@ -88,17 +106,80 @@ class UBXHandler:
         if self.__app.dialog(DLGTUBX) is not None:
             self.__app.dialog(DLGTUBX).update_pending(msg)
 
-        # if SPARTN config dialog is open, send CFG & ACKs there as well
+        # if SPARTN config dialog is open, send CFG & ACKs there
         if self.__app.dialog(DLGTSPARTN) is not None:
             self.__app.dialog(DLGTSPARTN).update_pending(msg)
 
-        # if Spectrumview widget exists, send NAKs there as well
-        if self.__app.frm_spectrumview is not None and msg.identity in (
-            "ACK-ACK",
-            "ACK-NAK",
-        ):
-            if msg.clsID == 6 and msg.msgID == 1:  # CFG-MSG
-                self.__app.frm_spectrumview.update_pending(msg)
+        # if Spectrumview or Sysmon widgets are active, send ACKSs there
+        if msg.identity in ("ACK-ACK", "ACK-NAK"):
+            wdgs = self.__app.widgets
+            for wdg in (WDGSYSMON, WDGSPECTRUM):
+                if wdgs[wdg][VISIBLE]:
+                    if msg.clsID == 6 and msg.msgID == 1:  # CFG-MSG
+                        getattr(self.__app, wdgs[wdg]["frm"]).update_pending(msg)
+
+    def _process_MONVER(self, msg: UBXMessage):
+        """
+        Process MON-VER & MON-HW sentences.
+
+        :param UBXMessage msg: UBX config message
+        """
+
+        if self.__app.dialog(DLGTUBX) is not None:
+            self.__app.dialog(DLGTUBX).update_pending(msg)
+
+    def _process_MON_SYS(self, data: UBXMessage):
+        """
+        Process MON-SYS sentences - System Monitor Information.
+
+        :param UBXMessage data: MON-SYS parsed message
+        """
+
+        sysdata = {}
+        sysdata["bootType"] = data.bootType
+        sysdata["cpuLoad"] = data.cpuLoad
+        sysdata["cpuLoadMax"] = data.cpuLoadMax
+        sysdata["memUsage"] = data.memUsage
+        sysdata["memUsageMax"] = data.memUsageMax
+        sysdata["ioUsage"] = data.ioUsage
+        sysdata["ioUsageMax"] = data.ioUsageMax
+        sysdata["runTime"] = data.runTime
+        sysdata["noticeCount"] = data.noticeCount
+        sysdata["warnCount"] = data.warnCount
+        sysdata["errorCount"] = data.errorCount
+        sysdata["tempValue"] = data.tempValue
+        self.__app.gnss_status.sysmon_data = sysdata
+
+    def _process_MON_COMMS(self, data: UBXMessage):
+        """
+        Process MON-COMMS sentences - Comms Port Information.
+
+        :param UBXMessage data: MON-COMMS parsed message
+        """
+
+        commsdata = {}
+        for i in range(1, data.nPorts + 1):
+            idx = f"_{i:02}"
+            pid = getattr(data, "portId" + idx)
+            tx = getattr(data, "txUsage" + idx)
+            txmax = getattr(data, "txPeakUsage" + idx)
+            txbytes = getattr(data, "txBytes" + idx)
+            txpending = getattr(data, "txPending" + idx)
+            rx = getattr(data, "rxUsage" + idx)
+            rxmax = getattr(data, "rxPeakUsage" + idx)
+            rxbytes = getattr(data, "rxBytes" + idx)
+            rxpending = getattr(data, "rxPending" + idx)
+            commsdata[pid] = (
+                tx,
+                txmax,
+                txbytes,
+                txpending,
+                rx,
+                rxmax,
+                rxbytes,
+                rxpending,
+            )
+        self.__app.gnss_status.comms_data = commsdata
 
     def _process_NAV_POSLLH(self, data: UBXMessage):
         """
@@ -113,6 +194,7 @@ class UBXHandler:
         self.__app.gnss_status.alt = data.hMSL / 1000  # meters
         self.__app.gnss_status.hacc = data.hAcc / 1000  # meters
         self.__app.gnss_status.vacc = data.vAcc / 1000  # meters
+        self.__app.gnss_status.sep = (data.height - data.hMSL) / 1000  # meters
 
     def _process_NAV_PVT(self, data: UBXMessage):
         """
@@ -131,6 +213,7 @@ class UBXHandler:
         self.__app.gnss_status.sip = data.numSV
         self.__app.gnss_status.speed = data.gSpeed / 1000  # m/s
         self.__app.gnss_status.track = data.headMot
+        self.__app.gnss_status.sep = (data.height - data.hMSL) / 1000  # meters
         self.__app.gnss_status.fix = fix2desc("NAV-PVT", data.fixType)
         if data.carrSoln > 0:
             self.__app.gnss_status.fix = fix2desc("NAV-PVT", data.carrSoln + 5)
@@ -139,6 +222,21 @@ class UBXHandler:
         if data.lastCorrectionAge != 0:
             self.__app.gnss_status.diff_age = corrage2int(data.lastCorrectionAge)
 
+    def _process_NAV_PVAT(self, data: UBXMessage):
+        """
+        Process NAV-PVAT sentence -  Navigation position velocity attitude time solution.
+
+        :param UBXMessage data: NAV-PVT parsed message
+        """
+
+        self.__app.gnss_status.utc = itow2utc(data.iTOW)  # datetime.time
+        self.__app.gnss_status.lat = data.lat
+        self.__app.gnss_status.lon = data.lon
+        self.__app.gnss_status.alt = data.hMSL / 1000  # meters
+        self.__app.gnss_status.speed = data.gSpeed / 1000  # m/s
+        self.__app.gnss_status.sip = data.numSV
+        self.__app.gnss_status.sep = (data.height - data.hMSL) / 1000  # meters
+
     def _process_NAV_VELNED(self, data: UBXMessage):
         """
         Process NAV-VELNED sentence - Velocity Solution in North East Down format.
@@ -162,7 +260,7 @@ class UBXHandler:
         """
 
         settings = self.__app.frm_settings.config
-        show_unused = settings["unusedsat"]
+        show_unused = settings["unusedsat_b"]
         self.gsv_data = []
         num_siv = int(data.numSvs)
 
@@ -196,6 +294,15 @@ class UBXHandler:
         if data.carrSoln > 0:
             self.__app.gnss_status.fix = fix2desc("NAV-STATUS", data.carrSoln + 5)
 
+    def _process_NAV_SVIN(self, data: UBXMessage):
+        """
+        Process NAV-SVIN sentences - Survey In Status.
+
+        :param UBXMessage data: NAV-SVIN parsed message
+        """
+
+        self.__app.svin_countdown(data.dur, data.valid, data.active)
+
     def _process_NAV_SVINFO(self, data: UBXMessage):
         """
         Process NAV-SVINFO sentences - Space Vehicle Information.
@@ -206,7 +313,7 @@ class UBXHandler:
         """
 
         settings = self.__app.frm_settings.config
-        show_unused = settings["unusedsat"]
+        show_unused = settings["unusedsat_b"]
         self.gsv_data = []
         num_siv = int(data.numCh)
 
@@ -245,6 +352,52 @@ class UBXHandler:
         self.__app.gnss_status.hdop = data.hDOP
         self.__app.gnss_status.vdop = data.vDOP
 
+    def _process_NAV_RELPOSNED(self, data: UBXMessage):
+        """
+        Process NAV-RELPOSNED sentence - Relative position of rover.
+
+        Two version of this message:
+        - version 0 has no heading or length attributes, so these
+          must be derived from the NED values
+        - version 1 has heading and length attributes
+
+        :param UBXMessage data: NAV-RELPOSNED parsed message
+        """
+
+        if data.version == 0x00:
+            n = data.relPosN + data.relPosHPN * 1e-2
+            e = data.relPosE + data.relPosHPE * 1e-2
+            d = data.relPosD + data.relPosHPD * 1e-2
+            relPosLength, relPosHeading = ned2vector(n, e, d)
+            n = data.accN * 1e-2
+            e = data.accE * 1e-2
+            d = data.accD * 1e-2
+            accLength, _ = ned2vector(n, e, d)
+            accHeading = accLength * relPosHeading / relPosLength  # ballpark
+        else:
+            relPosLength, relPosHeading, accLength, accHeading = (
+                data.relPosLength + data.relPosHPLength * 1e-2,
+                data.relPosHeading,
+                data.accLength * 1e-2,
+                data.accHeading,
+            )
+
+        self.__app.gnss_status.rel_pos_heading = relPosHeading
+        self.__app.gnss_status.rel_pos_length = relPosLength
+        self.__app.gnss_status.acc_heading = accHeading
+        self.__app.gnss_status.acc_length = accLength
+        self.__app.gnss_status.rel_pos_flags = [
+            data.gnssFixOK,
+            data.diffSoln,
+            data.relPosValid,
+            data.carrSoln,
+            data.isMoving,
+            data.refPosMiss,
+            data.refObsMiss,
+            data.relPosHeadingValid,
+            data.relPosNormalized,
+        ]
+
     def _process_HNR_PVT(self, data: UBXMessage):
         """
         Process HNR-PVT sentence -  High Rate Navigation position velocity time solution.
@@ -304,3 +457,13 @@ class UBXHandler:
 
         if self.__app.dialog(DLGTSPARTN) is not None:
             self.__app.dialog(DLGTSPARTN).update_pending(data)
+
+    def _process_RXM_PMP(self, data: UBXMessage):
+        """
+        Process RXM-PMP sentence - SPARTN L-Band data.
+
+        :param UBXMessage data: RXM-PMP message
+        """
+
+        if self.__app.dialog(DLGTSPARTN) is not None:
+            self.__app.dialog(DLGTSPARTN).update_pending(data)
